/* filename:    p_ac.g
** description: The program computes autocovariances
**              for the VAR 
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

/******************************************************************
**         Load Parameter Draws from MH Output
*/

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\" $+ llam $+ lsamp $+ "varpa"  ;

open fhpara = ^lpara for read;

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags) + nvar*(nvar+1)/2;

/* Initialize Output files
*/
opath     = resupath $+ "\\mhrun" $+ mhrun;
chkdir    = chk_dir(opath);

ovarac    = opath $+ "\\" $+ llam $+ lsamp $+ "ac";

create fhvarac = ^ovarac with AC, nvar+(acmax+1)*nvar^2, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;
nsim    = 0;

do until eofloop; 

   /* Read blocks of size nread
   */
   varparasim = readr( fhpara, nblock );
   nread      = rows(varparasim);
   acsim      = zeros(nread,nvar+(acmax+1)*nvar^2);

   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);
   
   j = 1;
   do until j > nread;
   
      if fmod(j,100)==0;".";; pause(1e-20); endif;

      /* Create Matrices CC and Sig
      */
      SIG_sim = xpnd(varparasim[j,nvar*(1+nvar*nlags)+1:nvarpara]');
      CCt_sim = reshape(varparasim[j,1:nvar*(1+nvar*nlags)]',nvar,1+nvar*nlags);
      CC_sim  = CCt_sim';
      
      /* Compute MA representation from companion form VAR
      */ 
      if nlags == 1;
        CCtilde  =  CCt_sim[.,1+1:1+nlags*nvar];
        CC0tilde = CCt_sim[.,1];
        SIGtilde = SIG_sim;
      else;
        CCtilde  =  CCt_sim[.,1+1:1+nlags*nvar] |
                   (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
        CC0tilde = CCt_sim[.,1] | zeros(nvar*(nlags-1),1);
        SIGtilde = (SIG_sim ~ zeros(nvar, nvar*(nlags-1)) )|
                   zeros(nvar*(nlags-1),nvar*nlags);
      endif;

      AC0tilde   = dlyap(CCtilde,Sigtilde);
      yybartilde = inv(eye(rows(CCtilde))-CCtilde)*CC0tilde;
      
      if nlags == 1;
         Msel = eye(nvar);
      else;
         Msel    = eye(nvar) | zeros((nlags-1)*nvar,nvar) ;
      endif;

      yybar = Msel'*yybartilde;
      
      AC0   = Msel'*AC0tilde*Msel;    
      AC0tr = sqrt(diag(AC0))*( sqrt(diag(AC0)) )';
      
      seldiag = eye(rows(AC0));
      seloffdiag = ones(rows(AC0),cols(AC0)) - seldiag;
      
      AC0 = seldiag.*sqrt(AC0tr) + seloffdiag.*( AC0./AC0tr);
      
      acall = yybar' ~ vec(AC0)';
      
      if acmax > 0;
      
         acind = 1;
         do until acind > acmax;
            AC0tilde = CCtilde*AC0tilde;
            AC0 = Msel'*AC0tilde*Msel;
            AC0 = AC0./AC0tr;
            acall = acall ~ vec(AC0)';
            acind = acind + 1;
         endo;
         
      endif;
      
      acsim[j,.] = acall;
            
      j = j+1;
  endo;

  wr = writer(fhvarac,acsim);
  loopct = loopct + 1;

endo;

closeall fhvarac, fhpara;
