/* filename:    irfplot.g
** description: The program plots the 
**              the VAR impulse response functions
** created:     11/06/01
*/

/********************************************************
** Open files with IRF Draw summaries
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

/******************************************************************
**         Load IRF summaries
*/

lpath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

lvarirf  = lpath $+ "\\" $+ llam $+ lsamp $+ "varirfs"; 
ldsgeirf = lpath $+ "\\" $+ llam $+ lsamp $+ "dsgeirfs"; 

ti    = seqa(1,1,nirf);

/* Read Posterior Simulation Summary File:
** DSGE model
*/
open fhdsgeirf = ^ldsgeirf for read;

dsgeirf_m = readr(fhdsgeirf,1);
dsgeirf_s = readr(fhdsgeirf,1);
dsgeirf_hpd_l = readr(fhdsgeirf,1);
dsgeirf_hpd_h = readr(fhdsgeirf,1);

dsgeirf_m     = reshape(dsgeirf_m,nvar^2,nirf)';
dsgeirf_hpd_l = reshape(dsgeirf_hpd_l,nvar^2,nirf)';
dsgeirf_hpd_h = reshape(dsgeirf_hpd_h,nvar^2,nirf)';

/* Read Posterior Simulation Summary File:
** VAR model
*/
open fhvarirf =  ^lvarirf for read;

varirf_m = readr(fhvarirf,1);
varirf_s = readr(fhvarirf,1);
varirf_hpd_l = readr(fhvarirf,1);
varirf_hpd_h = readr(fhvarirf,1);

varirf_m     = reshape(varirf_m,nvar^2,nirf)';
varirf_hpd_l = reshape(varirf_hpd_l,nvar^2,nirf)';
varirf_hpd_h = reshape(varirf_hpd_h,nvar^2,nirf)';

closeall fhdsgeirf, fhvarirf;

/* Plot Impulse Response functions
*/

titlestr = "Monetary Pol -> Output"  $|
           "Monetary Pol -> Inflation" $|
           "Monetary Pol -> Interest" $|
           "G-Shock -> Output" $|
           "G-Shock -> Inflation" $|
           "G-Shock -> Interest" $|
           "Z-Shock -> Output" $|
           "Z-Shock -> Inflation" $|
           "Z-Shock -> Interest";

ystr     = "Percent";

/*
drawtics = ( -0.2 ~ 0.1 ~ 0.1 )|
           ( -0.2 ~ 0.1 ~ 0.1 )|
           (  0 ~ 1 ~ 0.2 )|
           (  0   ~ 1 ~ 0.2 )|
           (  -.1   ~ .1 ~ 0.1 )|
           (  -.1   ~ .1 ~ 0.1 )|
           (  0   ~ 2 ~ 0.5   )|
           (  0   ~ 0.6 ~ 0.2   )|
           (  0   ~ 0.6 ~ 0.2   );
*/
drawtics = ( -1 ~ 1 ~ 0.5 )|
           ( -0.3 ~ 0.1 ~ 0.1 )|
           ( -0.2 ~ 1 ~ 0.2 )|
           (  -.5   ~ 2 ~ 0.5 )|
           (  -.5   ~ .5 ~ 0.25 )|
           (  -.5   ~ .5 ~ 0.25 )|
           (  0   ~ 3 ~ 1   )|
           (  -.2   ~ 0.8 ~ 0.2   )|
           (  -.2   ~ 0.8 ~ 0.2   );

           
graphset;
begwind;
margin(0,0,0.2,0.2);
window(nvar,nvar,0);
fonts("microb");
_ptitlht = 0.3;
_paxht = 0.3;
_pnumht = 0.25;
_pltype = {6 2 2 1};
_pcolor = {4 4 4 1};
_protate = 0; 
_plctrl = 0;
_plwidth = 5.5;
_psymsiz = 5;
_protate = 1;
_plegstr = "Mean\00090% HPD(U)\00090% HPD(L)\000DSGE";

pl_ind = 1;
do until pl_ind > nvar^2;

   title(titlestr[pl_ind]);
   ylabel(ystr);       
   ytics(drawtics[pl_ind,1],drawtics[pl_ind,2],drawtics[pl_ind,3],1); 
   xy(ti, varirf_m[.,pl_ind]~varirf_hpd_l[.,pl_ind]~varirf_hpd_h[.,pl_ind]~dsgeirf_m[.,pl_ind] );
   nextwind;
                 
   pl_ind = pl_ind+1;
endo;
    
endwind;
