/* filename:    vardraw.g
** description: The program generates draws from the VAR
**              conditional on the DSGE model draws
** created:     11/06/01
*/

/********************************************************
** Open files with Parameter Draws
*/

if lamind < 10;
   llam = "0" $+ ftos(lamind,"%*.*lf",1,0);
else;
   llam = ftos(lamind,"%*.*lf",2,0);
endif;

if Tind < 10;
   lsamp = "0" $+ ftos(Tind,"%*.*lf",1,0);
else;
   lsamp = ftos(Tind,"%*.*lf",2,0);
endif;

/******************************************************************
**         Load Parameter Draws from MH Output
*/

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\" $+ llam $+ lsamp $+ "pa"  ;

open fhpara = ^lpara for read;

/* Define some Constants
*/
nvarpara  = nvar*(1+nvar*nlags) + nvar*(nvar+1)/2;

/* Initialize Output files
*/
opath  = parapath $+ "\\mhrun" $+ mhrun;
ovarpara  = opath $+ "\\" $+ llam $+ lsamp $+ "varpa"  ;

create fhvarpara = ^ovarpara with VARPARA, nvarpara, 8;

/* Initialization
*/ 

eofloop = 0;
loopct  = 1;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nblock );
   nread   = rows(parasim);

   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara);
   lambda;
   
   varparasim = zeros(nread,nvarpara);
   
   j = 1;
   do until j > rows(parasim);
   
      if fmod(j,100)==0;".";; pause(1e-20); endif;

      /* For each para, draw a CC and Sig for the VAR
      */
      {yyyyd, xxyyd, xxxxd, retcode} = varprio(parasim[j,.]',msel);
      
      yyyyc = yyyy + lambda*nobs*yyyyd;
      xxyyc = xxyy + lambda*nobs*xxyyd;
      xxxxc = xxxx + lambda*nobs*xxxxd;

      nobsc = nobs + nobs*lambda;    
      k     = 1+nlags*nvar;
    
      /* Compute OLS estimates
      */
      CC_hat  = inv(XXXXc)*XXYYc;
      C_hat   = vec(CC_hat);
      SIG_hat = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc;

      val = 0;
      do until val == 1;

         /* draw from the marginal posterior of SIG
         */
         H = nobsc*Sig_hat;
         Z = chol(inv(H))'*rndn(nvar,nobsc-k);
         Sig_sim = inv(Z*Z');

         /* draw from the conditional posterior of C
         */
         VC     = Sig_sim.*.inv(XXXXc);
         C_sim  = C_hat + chol(VC)'*rndn(nvar*k,1);
         CCt_sim= reshape(C_sim,nvar,k);

         /* Check for stationarity
         */
         if nlags == 1;
            CCtilde = CCt_sim[.,1+1:k];
         else;
            CCtilde =  CCt_sim[.,1+1:k] |
                       (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
         endif;
         laCCtil = eig(CCtilde);
         if maxc(abs(laCCtil)) < 1; 
            val = 1;
         endif;

      endo; 

      varparasim[j,.] = vec(C_sim)' ~ vech(Sig_sim)';
      
      j = j+1;
  endo;
      
  /* Write Output
  */
  wr = writer(fhvarpara,varparasim);

  loopct = loopct + 1;

endo;

closeall fhvarpara, fhpara;
