/* filename:    varprior.g
** created:     11/05/01
*/

proc(4) = varprio(para,msel);
/********************************************************************************* 
** Global nvar, nlags
**
** description: The program converts the cross moment matrices for the DSGE prior
*********************************************************************************/

local TT,QQ,RR,HH,DD,ZZ,VV,RC,
      retcode,yyyyd,xxxxd,xxyyd,
      GAMM0,GA0,Gl,TTl,
      l,r;

yyyyd = 0;
xxyyd = 0;
xxxxd = 0;

/**********************************************************
**      System matrices for State-space
**********************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat(para,msel);

/**********************************************************
** Check determinacy 
**********************************************************/
if (RC[1] == 1) AND (RC[2]==1);
   /* determinacy */
   retcode = 1;

elseif (RC[1] == 1) AND (RC[2]==0) ;
   /* indeterminacy */
   retcode = 0;
   goto eofdsgeprio;

else;
   /* no equilibrium exists, numerical problems */
   retcode = 0;
   goto eofdsgeprio;

endif;


/**********************************************************
**  Compute nlags autocovariances
**********************************************************/

GAMM0 = zeros(nvar^2,nlags+1);

GA0 = dlyap(TT,RR*QQ*RR');

Gl   = ZZ*GA0*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;

GAMM0[.,1] = vecr(Gl);
 
TTl = TT;
l=1;
do until l > nlags;

   Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RR*VV);
   GAMM0[.,l+1] = vecr(Gl);           
   TTl = TTl*TT;
   l = l+1; 

endo;

/**********************************************************
**  Create limit cross product matrices
**********************************************************/

yyyyd = reshape(GAMM0[.,1],nvar,nvar) + DD*DD';
     
xxyyd = DD;
l = 1;
do until l > nlags;
   xxyyd = xxyyd ~ (reshape(GAMM0[.,l+1],nvar,nvar) + DD*DD');
   l = l+1;
endo;
xxyyd = xxyyd';
     
xxxxd = zeros(1+nlags*nvar,1+nlags*nvar);
xxxxd[1,1] = 1;
xxxxd[1,2:1+nlags*nvar] = ones(1,nlags).*.DD';
xxxxd[2:1+nlags*nvar,1] = ones(nlags,1).*.DD;
     
r = 1;
do until r>nlags;
     l = r;
     do until l>nlags;
        xxxxd[1+nvar*(r-1)+1:1+nvar*r , 1+nvar*(l-1)+1:1+nvar*l ] = reshape(GAMM0[.,l-r+1],nvar,nvar)+DD*DD';
        xxxxd[1+nvar*(l-1)+1:1+nvar*l , 1+nvar*(r-1)+1:1+nvar*r ] = reshape(GAMM0[.,l-r+1],nvar,nvar)'+DD*DD';
        l = l+1;
     endo;
     r = r+1;
endo;
     
eofdsgeprio:
     
retp(yyyyd,xxyyd,xxxxd,retcode);
endp;
     

proc(1) = marglh(para,msel,lambda);
/***************************************************************************
**     global
**     yyyy, xxxx, xxyy, nobs,
**     nlags, nvar,
**
**     Compute marginal likelihood
***************************************************************************/

local yyyyd, xxxxd, xxyyd,
      yyyyc, xxxxc, xxyyc,
      retcode,
      k, pyt, pytd, nobsc, lnkapt, lnkaptd,
      SIG_tilde, SIG_stard, PHI_star, i;

/* Compute cross product matrices for DSGE dummy observations
*/
{yyyyd, xxyyd, xxxxd, retcode} = varprio(para,msel);

if retcode == 0;
    pYT = -1E10;
    pYTd = -1E-10;
    goto eodsgepost;
endif;

if lambda < 101;

    yyyyc = yyyy + lambda*nobs*yyyyd;
    xxyyc = xxyy + lambda*nobs*xxyyd;
    xxxxc = xxxx + lambda*nobs*xxxxd;

    nobsc = nobs + nobs*lambda;    
    k     = 1+nlags*nvar;
    
    /* Compute OLS estimates for dummy observations
    */
    /* SIG_stard = ( YYYYd -XXYYd'*inv(XXXXd)*XXYYd ); */
    SIG_stard = ( YYYYd -XXYYd'*( XXYYd / XXXXd ) ); 
    
    lnkaptd = 0.5*nvar*(nobs*lambda-k)*ln(2);
    i = 1;
    do until i > nvar;
       lnkaptd = lnkaptd + lngam(0.5*(lambda*nobs-k+1-i));
       i=i+1;
    endo;
            
    pYtd  = lnkaptd  - 0.5*(lambda*nobs-k)*ln(det(lambda*nobs*Sig_stard)) - 0.5*nvar*ln(det(lambda*nobs*XXXXd));

    /* Compute OLS estimates for actual and dummy observations
    */
    /* SIG_tilde = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc; */
    SIG_tilde = ( YYYYc -XXYYc'*( XXYYc / XXXXc) )/nobsc;
    
    lnkapt = -0.5*nobs*nvar*ln(2*pi) + 0.5*nvar*(nobsc-k)*ln(2);
    i = 1;
    do until i > nvar;
       lnkapt = lnkapt + lngam(0.5*(nobsc-k+1-i));
       i = i+1;
    endo;
            
    pYt  = lnkapt  - 0.5*(nobsc-k)*ln(det(nobsc*Sig_tilde)) - 0.5*nvar*ln(det(XXXXc)) - pYtd;

else;

    /* Compute OLS estimates for dummy observations
    */
    SIG_stard = ( YYYYd -XXYYd'*inv(XXXXd)*XXYYd );
    PHI_star  = inv(XXXXd)*XXYYd;
        
    pyt = -0.5*nobs*nvar*ln(2*pi) - 0.5*nobs*ln(det(SIG_stard))
          -0.5*sumc(diag(inv(SIG_stard)*(yyyy - xxyy'*PHI_star - PHI_star'*xxyy + PHI_star'*xxxx*PHI_star)));          
    
endif;
    
eodsgepost:
    
retp(pYt);
endp;
