/*************************************************************************
**  Simulate the DSGE-VAR model 
**************************************************************************/

/* filename: var_sim.g
** created:  
** Modifications:
**
*/

new;

nobs = 200;
nvar = 3;
yyyy = 0;
xxxx = 0;
xxyy = 0;

/****************************************************
* Housecleaning and set defaults
****************************************************/

library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

_fcmptol = 1E-10;

#include nkmp_mod.src
#include nkmp_para.src
#include nkmp_spec.g
#include var_spec.g
#include pathspec.g
#include p_varprior.g
   
/* Load DSGE model parameters
*/

para = pmsv(msel);
npara = rows(para);

load path = ^priopath para[npara,3] = "msim_par.out";
para      = para[.,datasel-4];

/*************************************************
**            Draw some VAR parameters
*************************************************/

/** set seed **/
rndseed 1234;

lambda = 0.5;


{YYYYd, XXYYd, XXXXd, retcode} = varprio(para,msel);
      
nobsc = nobs*lambda;    
k     = 1+nlags*nvar;
    
/* Compute OLS estimates
*/
CC_hat  = inv(XXXXd)*XXYYd;
C_hat   = vec(CC_hat);
SIG_hat = ( YYYYd -XXYYd'*inv(XXXXd)*XXYYd );

val = 0;
do until val == 1;

         /* draw from the marginal posterior of SIG
         */
         H = nobsc*Sig_hat;
         Z = chol(inv(H))'*rndn(nvar,nobsc-k);
         Sig_sim = inv(Z*Z');

         /* draw from the conditional posterior of C
         */
         VC     = Sig_sim.*.inv(XXXXd)/nobsc;
         C_sim  = C_hat + chol(VC)'*rndn(nvar*k,1);
         CCt_sim= reshape(C_sim,nvar,k);
         CC_sim = CCt_sim';

         /* Check for stationarity
         */
         if nlags == 1;
            CCtilde = CCt_sim[.,1+1:k];
         else;
            CCtilde =  CCt_sim[.,1+1:k] |
                       (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));
         endif;
         laCCtil = eig(CCtilde);
         if maxc(abs(laCCtil)) < 1; 
            val = 1;
         endif;

endo; 

/*************************************************
**            Draw observations from VAR
*************************************************/

/** set seed **/
rndseed 1234;

ytsim = zeros(Tsimtotal,nvar);
xt    = 1 | zeros(k-1,1); 
SIG_chol = chol(SIG_sim);

t = 2;
do until t > Tsimtotal;

   ytsim[t,.] = xt'*CC_sim + rndn(1,3)*SIG_chol;
   xt[1+nvar+1:k] = xt[1+1:k-nvar];
   xt[1+1:1+nvar] = ytsim[t,.]';

   if fmod(t,1000)==0;".";; pause(1e-20); endif;   

   t = t+1;

endo;

yt = ytsim[Tsimburn+1:Tsimtotal,.];

"Data Simulated Successfully";
"Parameter Choice:    " datasel;
"Model Specification: " msel;
"Parameters (theta):  "; para;
"Mean of YT:          " meanc(yT)';
"StD of YT:           " stdc(yT)';

eofsunsim:

/********************************************************
** Save simulated data
*/

osimdata         = datapath $+ "\\" $+ lmodel $+ "sim" $+ dataselstr;
create fhsimdata = ^osimdata with SERIES, cols(yt), 8;
wr               = writer(fhsimdata,yT);
closeall fhsimdata;   

"Data saved";
